#' BFI, Grit, and Hope data from Bainbridge, Ludeke, & Smillie (2022),
#' Study 2b.
#'
#' The subset of the data reported in the paper include only the BFI-2, Grit,
#' and hope.
#'
#' @format ## `BFIGritHopeImp`
#' A data frame with 388 rows and 95 columns:
#' \describe{
#'   \item{bfi_e1_1}{BFI-2 Extraversion: Sociability, Item 1}
#'   \item{bfi_e1_2}{BFI-2 Extraversion: Sociability, Item 2}
#'   \item{bfi_e1_3}{BFI-2 Extraversion: Sociability, Item 3}
#'   \item{bfi_e1_4}{BFI-2 Extraversion: Sociability, Item 4}
#'   \item{bfi_e2_1}{BFI-2 Extraversion: Assertiveness, Item 1}
#'   \item{bfi_e2_2}{BFI-2 Extraversion: Assertiveness, Item 2}
#'   \item{bfi_e2_3}{BFI-2 Extraversion: Assertiveness, Item 3}
#'   \item{bfi_e2_4}{BFI-2 Extraversion: Assertiveness, Item 4}
#'   \item{bfi_e3_1}{BFI-2 Extraversion: Energy Level, Item 1}
#'   \item{bfi_e3_2}{BFI-2 Extraversion: Energy Level, Item 2}
#'   \item{bfi_e3_3}{BFI-2 Extraversion: Energy Level, Item 3}
#'   \item{bfi_e3_4}{BFI-2 Extraversion: Energy Level, Item 4}
#'   \item{bfi_a1_1}{BFI-2 Agreeableness: Compassion, Item 1}
#'   \item{bfi_a1_2}{BFI-2 Agreeableness: Compassion, Item 2}
#'   \item{bfi_a1_3}{BFI-2 Agreeableness: Compassion, Item 3}
#'   \item{bfi_a1_4}{BFI-2 Agreeableness: Compassion, Item 4}
#'   \item{bfi_a2_1}{BFI-2 Agreeableness: Respectfulness, Item 1}
#'   \item{bfi_a2_2}{BFI-2 Agreeableness: Respectfulness, Item 2}
#'   \item{bfi_a2_3}{BFI-2 Agreeableness: Respectfulness, Item 3}
#'   \item{bfi_a2_4}{BFI-2 Agreeableness: Respectfulness, Item 4}
#'   \item{bfi_a3_1}{BFI-2 Agreeableness: Trust, Item 1}
#'   \item{bfi_a3_2}{BFI-2 Agreeableness: Trust, Item 2}
#'   \item{bfi_a3_3}{BFI-2 Agreeableness: Trust, Item 3}
#'   \item{bfi_a3_4}{BFI-2 Agreeableness: Trust, Item 4}
#'   \item{bfi_c1_1}{BFI-2 Conscientiousness: Organization, Item 1}
#'   \item{bfi_c1_2}{BFI-2 Conscientiousness: Organization, Item 2}
#'   \item{bfi_c1_3}{BFI-2 Conscientiousness: Organization, Item 3}
#'   \item{bfi_c1_4}{BFI-2 Conscientiousness: Organization, Item 4}
#'   \item{bfi_c2_1}{BFI-2 Conscientiousness: Productiveness, Item 1}
#'   \item{bfi_c2_2}{BFI-2 Conscientiousness: Productiveness, Item 2}
#'   \item{bfi_c2_3}{BFI-2 Conscientiousness: Productiveness, Item 3}
#'   \item{bfi_c2_4}{BFI-2 Conscientiousness: Productiveness, Item 4}
#'   \item{bfi_c3_1}{BFI-2 Conscientiousness: Responsibility, Item 1}
#'   \item{bfi_c3_2}{BFI-2 Conscientiousness: Responsibility, Item 2}
#'   \item{bfi_c3_3}{BFI-2 Conscientiousness: Responsibility, Item 3}
#'   \item{bfi_c3_4}{BFI-2 Conscientiousness: Responsibility, Item 4}
#'   \item{bfi_n1_1}{BFI-2 Negative Emotionality: Anxiety, Item 1}
#'   \item{bfi_n1_2}{BFI-2 Negative Emotionality: Anxiety, Item 2}
#'   \item{bfi_n1_3}{BFI-2 Negative Emotionality: Anxiety, Item 3}
#'   \item{bfi_n1_4}{BFI-2 Negative Emotionality: Anxiety, Item 4}
#'   \item{bfi_n2_1}{BFI-2 Negative Emotionality: Depression, Item 1}
#'   \item{bfi_n2_2}{BFI-2 Negative Emotionality: Depression, Item 2}
#'   \item{bfi_n2_3}{BFI-2 Negative Emotionality: Depression, Item 3}
#'   \item{bfi_n2_4}{BFI-2 Negative Emotionality: Depression, Item 4}
#'   \item{bfi_n3_1}{BFI-2 Negative Emotionality: Emotional Volatility, Item 1}
#'   \item{bfi_n3_2}{BFI-2 Negative Emotionality: Emotional Volatility, Item 2}
#'   \item{bfi_n3_3}{BFI-2 Negative Emotionality: Emotional Volatility, Item 3}
#'   \item{bfi_n3_4}{BFI-2 Negative Emotionality: Emotional Volatility, Item 4}
#'   \item{bfi_o1_1}{BFI-2 Open-Mindedness: Intellectual Curiosity, Item 1}
#'   \item{bfi_o1_2}{BFI-2 Open-Mindedness: Intellectual Curiosity, Item 2}
#'   \item{bfi_o1_3}{BFI-2 Open-Mindedness: Intellectual Curiosity, Item 3}
#'   \item{bfi_o1_4}{BFI-2 Open-Mindedness: Intellectual Curiosity, Item 4}
#'   \item{bfi_o2_1}{BFI-2 Open-Mindedness: Aesthetic Sensitivity, Item 1}
#'   \item{bfi_o2_2}{BFI-2 Open-Mindedness: Aesthetic Sensitivity, Item 2}
#'   \item{bfi_o2_3}{BFI-2 Open-Mindedness: Aesthetic Sensitivity, Item 3}
#'   \item{bfi_o2_4}{BFI-2 Open-Mindedness: Aesthetic Sensitivity, Item 4}
#'   \item{bfi_o3_1}{BFI-2 Open-Mindedness: Creative Imagination, Item 1}
#'   \item{bfi_o3_2}{BFI-2 Open-Mindedness: Creative Imagination, Item 2}
#'   \item{bfi_o3_3}{BFI-2 Open-Mindedness: Creative Imagination, Item 3}
#'   \item{bfi_o3_4}{BFI-2 Open-Mindedness: Creative Imagination, Item 4}
#'   \item{grit_c_1}{Grit Consistency of Interests, Item 1}
#'   \item{grit_c_2}{Grit Consistency of Interests, Item 2}
#'   \item{grit_c_3}{Grit Consistency of Interests, Item 3}
#'   \item{grit_c_4}{Grit Consistency of Interests, Item 4}
#'   \item{grit_c_5}{Grit Consistency of Interests, Item 5}
#'   \item{grit_c_6}{Grit Consistency of Interests, Item 6}
#'   \item{grit_p_1}{Grit Persistence, Item 1}
#'   \item{grit_p_2}{Grit Persistence, Item 2}
#'   \item{grit_p_3}{Grit Persistence, Item 3}
#'   \item{grit_p_4}{Grit Persistence, Item 4}
#'   \item{grit_p_5}{Grit Persistence, Item 5}
#'   \item{grit_p_6}{Grit Persistence, Item 6}
#'   \item{hope_a_1}{Hope Agency, Item 1}
#'   \item{hope_a_2}{Hope Agency, Item 2}
#'   \item{hope_a_3}{Hope Agency, Item 3}
#'   \item{hope_a_4}{Hope Agency, Item 4}
#'   \item{hope_p_1}{Hope Pathways, Item 1}
#'   \item{hope_p_2}{Hope Pathways, Item 2}
#'   \item{hope_p_3}{Hope Pathways, Item 3}
#'   \item{hope_p_4}{Hope Pathways, Item 4}
#' }
#' @source <https://osf.io/f9hmg/files/eu8p9>
"BFIGritHope"
